//---------------------------------------------------------------------------
#include <math.h>
#include <vcl.h>
#pragma hdrstop
#include "TGameBox.h"
#include "TGameTimer.h"
#include "TGameField.h"
#include "TGamePad.h"
#include "TGameSound.h"
#include "Common.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------
static const AnsiString STR_INSTRUCTIONS =
	"Baum Game - Instructions\r\n"
	"\r\n"
	"Get baumkuchens and bring them\r\n"
	"back home. One point per one.\r\n"
	"\r\n"
	"It is game over when there is no\r\n"
	"way to go around the player.";
//---------------------------------------------------------------------------
inline static int __fastcall Max(int AValue1, int AValue2)
{
	return AValue1 > AValue2 ? AValue1 : AValue2;
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// TGameBoxOptions
//---------------------------------------------------------------------------
__fastcall TGameBoxOptions::TGameBoxOptions() : HighScore(0), BusyGame(false), Sound(true)
{
	// do nothing
}
//---------------------------------------------------------------------------
__fastcall TGameBoxOptions::~TGameBoxOptions()
{
	// do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TGameBoxOptions::operator==(const TGameBoxOptions& ASrc) const
{
	return HighScore == ASrc.HighScore &&
		BusyGame == ASrc.BusyGame &&
		Sound == ASrc.Sound;
}
//---------------------------------------------------------------------------
bool __fastcall TGameBoxOptions::operator!=(const TGameBoxOptions& ASrc) const
{
	return !(ASrc == *this);
}
//---------------------------------------------------------------------------


                                                                       

//---------------------------------------------------------------------------
// class TGameBox : public TPaintBox
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TGameBox::SetMode(MODE AMode)
{
	FMode = AMode;

	if(FMode == MODE_TITLE) {
		DrawTitle(FBitmap->Canvas);
	}
	else if(FMode == MODE_NEW1P || FMode == MODE_NEW2P) {
		FField->Prepare(FMode == MODE_NEW1P ? 1 : 2);
	}
}
//---------------------------------------------------------------------------
TGameBoxOptions __fastcall TGameBox::GetOptions()
{
	TGameBoxOptions ret;
	ret.HighScore = FField->HighScore;
	ret.BusyGame = FField->BusyGame;
	ret.Sound = FSound->Enabled;

	return ret;
}
//---------------------------------------------------------------------------
void __fastcall TGameBox::SetOptions(TGameBoxOptions AOptions)
{
	if(AOptions == Options) return;

	FField->HighScore = AOptions.HighScore;
	FField->BusyGame = AOptions.BusyGame;
	FSound->Enabled = AOptions.Sound;
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TGameBox::SetImages(TImageList* AImages)
{
	if(AImages == FImages) return;

	FImages = AImages;
	DoChange();
}
//---------------------------------------------------------------------------
int __fastcall TGameBox::GetOriginalWidth()
{
	return FBitmap->Width;
}
//---------------------------------------------------------------------------
int __fastcall TGameBox::GetOriginalHeight()
{
	return FBitmap->Height;
}
//---------------------------------------------------------------------------
void __fastcall TGameBox::TimerTimer(TObject *Sender)
{
	for(int i = 0; i < MAXPLAYERCOUNT; i++)
		FPads[i]->Update();

	switch(FMode) {
		MODE_NEW1P:
		MODE_NEW2P:
			FField->Move(FPads, FSound);
			FField->Draw(FBitmap->Canvas, FImages);
			FField->Draw(FBitmap->Canvas, FImages);
			if(FField->GameOver) Mode = MODE_GAMEOVER;
			break;
	}

	for(int i = 0; i < MAXPLAYERCOUNT; i++)
		FPads[i]->Clear();

	Paint();
}
//---------------------------------------------------------------------------
void __fastcall TGameBox::DrawTitle(TCanvas* ACanvas)
{
	// draw background
	ACanvas->Brush->Color = static_cast<TColor>(RGB(0, 160, 0));
	ACanvas->FillRect(SCRR);

	// draw marks
	{
		int count = 6;
		double degree = random(360);
		int x, y;
		for(int i = 0; i < count; i++) {
			x = SCRR.Width() / 2 + 90 * cos(degree * M_PI / 180);
			y = SCRR.Height() / 2 + 90 * sin(degree * M_PI / 180);
			Images->Draw(ACanvas, (x - UNITW / 2), (y - UNITH / 2), IMAGETABLE[30]);
			degree += 360 / count;
		}
	}


	// draw lines
	{
		// preapre size
		TStringList* lines = new TStringList();
		lines->Text = STR_INSTRUCTIONS;
		int lineWidth = 0;
		int lineHeight = 0;
		for(int i = 0; i < lines->Count; i++) {
			lineWidth = Max(lineWidth, ACanvas->TextWidth(lines->Strings[i]));
			lineHeight = Max(lineHeight, ACanvas->TextHeight(lines->Strings[i]));
		}

		// draw
		ACanvas->Brush->Style = bsClear;
		ACanvas->Font->Color = static_cast<TColor>(RGB(255, 255, 255));
		int dx = (SCRR.Width() - lineWidth) / 2;
		int dy = (SCRR.Height() - lineHeight * lines->Count) / 2;
		for(int i = 0; i < lines->Count; i++) {
			ACanvas->TextOut(dx, dy, lines->Strings[i]);
			dy += lineHeight;
		}

		delete lines;
	}
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
void __fastcall TGameBox::DoChange()
{
	if(FOnChange == NULL) return;

	FOnChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TGameBox::Paint()
{
	if(Parent == NULL || FBitmap == NULL) return;

	if(Width == FBitmap->Width && Height == FBitmap->Height)
		Canvas->Draw(0, 0, FBitmap);
	else
		Canvas->CopyRect(TRect(0, 0, Width, Height), FBitmap->Canvas,
			TRect(0, 0, FBitmap->Width, FBitmap->Height));
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TGameBox::TGameBox(TComponent* AOwner) : TPaintBox(AOwner), FMode(MODE_NONE),
	FTimer(NULL), FField(NULL), FSound(NULL), FBitmap(NULL), FImages(NULL), FOnChange(NULL)
{
	randomize();
                                                                      
	Width = SCRR.Width();
	Height = SCRR.Height();

	// prepare timer
	FTimer = new TGameTimer(100, &TimerTimer);

	// prepare field
	FField = new TGameField(GRIDW, GRIDH);

	// prepare pads
	FPads = new TGamePad*[MAXPLAYERCOUNT];
	for(int i = 0; i < MAXPLAYERCOUNT; i++) {
		FPads[i] = new TGamePad();
		FPads[i]->Count = PADIDCOUNT;
		for(int j = 0; j < PADIDCOUNT; j++)
			FPads[i]->Keys[j] = PADKEYS[i][j];
	}

	// prepare sound
	FSound = new TGameSound();
	TStringList* list = new TStringList();
	for(int i = 0; i < SOUNDIDCOUNT; i++)
		list->Add(SOUNDPATH + SOUNDFILENAMES[i]);
	FSound->LoadFromFile(list);
	delete list;

	// prepare off-screen bitmap
	FBitmap = new Graphics::TBitmap();
	FBitmap->Width = Width;
	FBitmap->Height = Height;
	FBitmap->Canvas->Font->Charset = DEFAULT_CHARSET;
	FBitmap->Canvas->Font->Name = "Arial";
	FBitmap->Canvas->Font->Height = -17;
	FBitmap->Canvas->Font->Style = FBitmap->Canvas->Font->Style << fsBold;

	Options = TGameBoxOptions();

	// start timer
	FTimer->Resume();
}
//---------------------------------------------------------------------------
__fastcall TGameBox::~TGameBox()
{
	Quit();

	try {
		FTimer->Terminate();
		FTimer->WaitFor();
	}
	catch(...) {}

	delete FBitmap;
	delete FSound;
	for(int i = 0; i < MAXPLAYERCOUNT; i++)
		delete FPads[i];
	delete [] FPads;
	delete FField;
	delete FTimer;
}
//---------------------------------------------------------------------------
void __fastcall TGameBox::Title()
{
	Mode = MODE_TITLE;
}
//---------------------------------------------------------------------------
void __fastcall TGameBox::New(int APlayerCount)
{
	switch(APlayerCount) {
		case 1: Mode = MODE_NEW1P; break;
		case 2: Mode = MODE_NEW2P; break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TGameBox::Quit()
{
	Mode = MODE_QUIT;
}
//---------------------------------------------------------------------------
void __fastcall TGameBox::KeyDown(int AKey)
{
	for(int i = 0; i < MAXPLAYERCOUNT; i++)
		FPads[i]->KeyDown(AKey);

	switch(Mode) {
		MODE_TITLE:
		MODE_GAMEOVER:
			if(AKey == VK_SPACE || AKey == VK_RETURN) New(1);
			break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TGameBox::KeyUp(int AKey)
{
	for(int i = 0; i < MAXPLAYERCOUNT; i++)
		FPads[i]->KeyUp(AKey);
}
//---------------------------------------------------------------------------

