//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TGameField.h"
#include "TGameMap.h"
#include "TGameUnit.h"
#include "TGamePad.h"
#include "TGameSound.h"
#include "Common.h"
#pragma package(smart_init)                                 
using namespace std;
//---------------------------------------------------------------------------
// class TGameField
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TGameField::Clear()
{
	for(unsigned int i = 0; i < FUnits.size(); i++)
		delete FUnits[i];
	FUnits.clear();
	FPlayerUnits.clear();
	FMap->Clear();
	FGameOver = false;
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TGameField::TGameField(int AWidth, int AHeight) :
	FMap(NULL), FHighScore(0), FBusyGame(false)
{
	FMap = new TGameMap(AWidth, AHeight);
}
//---------------------------------------------------------------------------
__fastcall TGameField::~TGameField()
{
	Clear();

	delete FMap;
}
//---------------------------------------------------------------------------
void __fastcall TGameField::RegisterUnit(TGameUnit* AUnit)
{
	FUnits.push_back(AUnit);
}
//---------------------------------------------------------------------------
void __fastcall TGameField::Prepare(int APlayerCount)
{
	// clear all
	Clear();

	// put trees
	for(int x = 0; x < FMap->Width; x++) {
		for(int y = 0; y < FMap->Height; y++) {
			if(x == 0 || x == FMap->Width - 1 || y == 0 || y == FMap->Height - 1)
				FMap->Walls[x][y] = GAMEWALL_TREE;
		}
	}

	// put players
	{
		TGamePlayerUnit* unit;
		int x, y;
		for(int i = 0; i < APlayerCount; i++) {
			unit = new TGamePlayerUnit(i);
			do {
				if(APlayerCount == 1) {
					x = FMap->Width / 2;
					y = FMap->Height / 2;
				}
				else {
					x = random(FMap->Width);
					y = random(FMap->Height);
				}
			} while(!FMap->IsEmpty(x, y));
			unit->X = x;
			unit->Y = y;
			FMap->Units[unit->X][unit->Y] = unit;
			RegisterUnit(unit);
			FPlayerUnits.push_back(unit);
		}
	}

	// put a home
	if(APlayerCount == 1) {
		TGameHomeUnit* unit = new TGameHomeUnit();
		unit->X = FMap->Width / 2;
		unit->Y = FMap->Height / 2;
		FMap->Units[unit->X][unit->Y] = unit;
		RegisterUnit(unit);
	}

	// put blocks
	{
		int x, y;
		for(int i = 0; i < 4; i++) {
			do {
				x = random(FMap->Width);
				y = random(FMap->Height);
			} while(!FMap->IsEmpty(x, y));
			FMap->Walls[x][y] = GAMEWALL_BLOCK;
		}
	}

	Status = "Started";
}
//---------------------------------------------------------------------------
void __fastcall TGameField::Move(TGamePad** APads, TGameSound* ASound)
{
	if(GameOver) return;

	// move units
	for(unsigned int i = 0; i < FUnits.size(); i++) {
		// delete if invalid unit
		if(!FUnits[i]->Valid) {
			delete FUnits[i];
			FUnits.erase(FUnits.begin() + i--);
			continue;
		}
		// move unit
		FUnits[i]->Move(this, APads, ASound);
	}

	// put a unit
	if(random(BusyGame ? 3 : 6) == 0) {
		int x, y;
		TGameUnit* unit;
		for(int i = 0; i < 6; i++) {
			x = random(FMap->Width);
			y = random(FMap->Height);
			if(!FMap->IsEmpty(x, y)) continue;
			unit = new TGameTargetUnit(random(6) == 0 ?
				TGameTargetUnit::TYPE_GELATO : TGameTargetUnit::TYPE_CAKE);
			unit->X = x;
			unit->Y = y;
			FMap->Units[unit->X][unit->Y] = unit;
			RegisterUnit(unit);
			break;
		}
	}

	// check game over
	if(FPlayerUnits.size() == 1) {
		if(FPlayerUnits[0]->Back) {
			FGameOver = true;
			Status = "Back Home";
			ASound->Play(SOUNDID_BACK);
		}
		else if(FPlayerUnits[0]->Stuck) {
			FGameOver = true;
			Status = "Game Over";
			ASound->Play(SOUNDID_GAMEOVER);
		}
		else return;
	}
	else {
		int flags = (FPlayerUnits[0]->Stuck ? 1 : 0) | (FPlayerUnits[1]->Stuck ? 2 : 0);
		if(flags != 0) {
			FGameOver = true;
			switch(flags) {
				case 1: Status = "P1 is Stuck"; break;
				case 2: Status = "P2 is Stuck"; break;
				case 3: Status = "Both are Stuck"; break;
			}
			ASound->Play(SOUNDID_GAMEOVER);
		}
		else return;
	}

	// update high-score
	for(unsigned int i = 0; i < FPlayerUnits.size(); i++) {
		if(FPlayerUnits[i]->Score > FHighScore)
			FHighScore = FPlayerUnits[i]->Score;
	}
}
//---------------------------------------------------------------------------
void __fastcall TGameField::Draw(TCanvas* ACanvas, TImageList* AImages)
{
	// draw changes on map
	{
		ACanvas->Brush->Color = static_cast<TColor>(RGB(0, 232, 0));
		ACanvas->Brush->Style = bsSolid;

		TPoint point;
		int image;
		for(int i = 0; i < FMap->ChangeCount; i++) {
			// get point to draw
			point = FMap->Changes[i];

			// draw background
			ACanvas->FillRect(TRect(UNITW * point.x, UNITH * point.y,
				UNITW * (point.x + 1), UNITH * (point.y + 1)));

			// draw image
			image = -1;
			switch(FMap->Walls[point.x][point.y]) {
				case GAMEWALL_TREE: image = 0; break;
				case GAMEWALL_BLOCK: image = 1; break;
				case GAMEWALL_TAIL: image = 2; break;
			}
			if(FMap->Units[point.x][point.y] != NULL) {
				image = FMap->Units[point.x][point.y]->Image;
			}
			if(image >= 0)
				AImages->Draw(ACanvas, UNITW * point.x, UNITH * point.y, IMAGETABLE[image]);
		}

		FMap->ClearChanges();
	}

	// draw status
	{
		// draw background
		ACanvas->Brush->Color = static_cast<TColor>(RGB(0, 160, 0));
		ACanvas->Brush->Style = bsSolid;
		ACanvas->Font->Color = static_cast<TColor>(RGB(255, 255, 255));
		ACanvas->FillRect(STATUSR);

		// draw message
		AnsiString text = " " + FStatus;
		ACanvas->TextOut(STATUSR.left,
			STATUSR.top + (STATUSR.Height() - ACanvas->TextHeight(text)) / 2, text);

		// draw score and high-score
		switch(FPlayerUnits.size()) {
			case 1: text = AnsiString("Sc: ") + FPlayerUnits[0]->Score +
				"  Hi: " + FHighScore;
				break;

			case 2: text = AnsiString("P1: ") + FPlayerUnits[0]->Score +
				"  P2: " + FPlayerUnits[1]->Score;
				break;
		}
		text += " ";
		ACanvas->TextOut(STATUSR.right - ACanvas->TextWidth(text),
			STATUSR.top + (STATUSR.Height() - ACanvas->TextHeight(text)) / 2, text);
	}
}
//---------------------------------------------------------------------------

