//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <IniFiles.hpp>
#include "TGameForm.h"
#include "TGameBox.h"
#pragma package(smart_init)
#pragma resource "*.dfm"
TGameForm *GameForm;
//---------------------------------------------------------------------------
static const AnsiString STR_TITLE = "Baum Game";
static const AnsiString STR_ABOUT =
	STR_TITLE + " for Win32 Version 1.0.1\r\n\r\n"
	"Copyright (C) 2003 Moochos. All Rights Reserved.";
static const AnsiString STR_TIPS =
	"[Controls]\r\n"
	"Player 1: Arrow keys\r\n"
	"Player 2: A, D, W and S keys\r\n"
	"\r\n"
	"[Tips]\r\n"
	"To start the game, press Space or Enter key.";
static const AnsiString FILE_HELP = ExtractFilePath(Application->ExeName) + "Help\\index.html";
static const AnsiString FILE_INI = ExtractFilePath(Application->ExeName) + "Settings.ini";
static const AnsiString INI_SECTION = "Settings";
static const AnsiString INI_KEY = "Values";
//---------------------------------------------------------------------------
// struct TGameFormOptions
//---------------------------------------------------------------------------
__fastcall TGameFormOptions::TGameFormOptions() : Sizable(false), LastPosition(true)
{
	// do nothing
}
//---------------------------------------------------------------------------
__fastcall TGameFormOptions::~TGameFormOptions()
{
	// do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TGameFormOptions::operator==(const TGameFormOptions& ASrc) const
{
	return Sizable == ASrc.Sizable &&
		LastPosition == ASrc.LastPosition;
}
//---------------------------------------------------------------------------
bool __fastcall TGameFormOptions::operator!=(const TGameFormOptions& ASrc) const
{
	return !(ASrc == *this);
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// published:
//---------------------------------------------------------------------------
void __fastcall TGameForm::FormKeyDown(TObject *Sender, WORD &Key, TShiftState Shift)
{
	FGameBox->KeyDown(Key);
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::FormKeyUp(TObject *Sender, WORD &Key, TShiftState Shift)
{
	FGameBox->KeyUp(Key);
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::MenuItemClick(TObject *Sender)
{
	if(Sender == New1Player_MenuItem) {
		FGameBox->New(1);
	}
	else if(Sender == New2Players_MenuItem) {
		FGameBox->New(2);
	}
	else if(Sender == Status_MenuItem) {
		Application->MessageBox((AnsiString("High-Score: ") +
			GameBox->Options.HighScore).c_str(), "Status", MB_ICONINFORMATION);
	}
	else if(Sender == ClearHighScore_MenuItem || Sender == BusyGame_MenuItem ||
		Sender == Sound_MenuItem) {
		TGameBoxOptions options = FGameBox->Options;
		if(Sender == ClearHighScore_MenuItem) {
			if(Application->MessageBox("Clearing high-score.\r\n\r\nSure?",
				Application->Title.c_str(), MB_OKCANCEL | MB_ICONQUESTION) != IDOK) return;
			options.HighScore = 0;
		}
		else if(Sender == BusyGame_MenuItem) {
			options.BusyGame = !options.BusyGame;
		}
		else if(Sender == Sound_MenuItem) {
			options.Sound = !options.Sound;
		}
		FGameBox->Options = options;
	}
	else if(Sender == Sizable_MenuItem || Sender == LastPosition_MenuItem) {
		TGameFormOptions options = Options;
		if(Sender == Sizable_MenuItem) {
			options.Sizable = !options.Sizable;
		}
		else if(Sender == LastPosition_MenuItem) {
			options.LastPosition = !options.LastPosition;
		}
		Options = options;
	}
	else if(Sender == ResizeTo100_MenuItem) {
		ResizeClient(100);
	}
	else if(Sender == ResizeTo150_MenuItem) {
		ResizeClient(150);
	}
	else if(Sender == ResizeTo200_MenuItem) {
		ResizeClient(200);
	}
	else if(Sender == Default_MenuItem) {
		if(Application->MessageBox("Applying default settings.\r\n\r\n"
			"High-score will not be cleared.\r\n\r\nSure?", Application->Title.c_str(),
			MB_OKCANCEL | MB_ICONQUESTION) != IDOK) return;
		DefaultSettings();
	}
	else if(Sender == Exit_MenuItem) {
		Close();
	}
	else if(Sender == Help2_MenuItem) {
		// confirm open
		if(Application->MessageBox("Opening a help file in HTML format.\r\n\r\nSure?",
			Application->Title.c_str(), MB_OKCANCEL | MB_ICONQUESTION) != IDOK) return;

		// open help file
		if(reinterpret_cast<int>(ShellExecute(Application->Handle, NULL,
			FILE_HELP.c_str(), NULL, NULL, SW_SHOWNORMAL)) > 32) return;

		// show error message
		Application->MessageBox(("Failed to open the file: "
			+ FILE_HELP).c_str(), Application->Title.c_str(), MB_ICONEXCLAMATION);
	}
	else if(Sender == Tips_MenuItem) {
		Application->MessageBox(STR_TIPS.c_str(), "Tips", MB_ICONINFORMATION);
	}
	else if(Sender == About_MenuItem) {
		Application->MessageBox(STR_ABOUT.c_str(), "About", MB_ICONINFORMATION);
	}
	else if(Sender == New1PlayerP_MenuItem) {
		New1Player_MenuItem->Click();
	}
	else if(Sender == New2PlayersP_MenuItem) {
		New2Players_MenuItem->Click();
	}
	else if(Sender == StatusP_MenuItem) {
		Status_MenuItem->Click();
	}
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::OptionsChange(TObject* ASender)
{
	int width = ClientWidth;
	int height = ClientHeight;
	BorderIcons = FOptions.Sizable ? BorderIcons << biMaximize : BorderIcons >> biMaximize;
	BorderStyle = FOptions.Sizable ? bsSizeable : bsSingle;
	ResizeClient(width, height);

	Sizable_MenuItem->Checked = Options.Sizable;
	LastPosition_MenuItem->Checked = Options.LastPosition;
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::GameBoxChange(TObject* ASender)
{
	BusyGame_MenuItem->Checked = FGameBox->Options.BusyGame;
	Sound_MenuItem->Checked = FGameBox->Options.Sound;
}
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TGameForm::SetOptions(TGameFormOptions AOptions)
{
	if(AOptions == FOptions) return;

	FOptions = AOptions;
	OptionsChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::ResizeClient(int APercentage)
{
	ResizeClient(GameBox->OriginalWidth * APercentage / 100,
		GameBox->OriginalHeight * APercentage / 100);
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::ResizeClient(int AWidth, int AHeight)
{
	ClientWidth = AWidth;
	ClientHeight = AHeight;
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::DefaultSettings()
{
	Options = TGameFormOptions();
	FGameBox->Options = TGameBoxOptions();
	ResizeClient(100);
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::LoadSettings()
{
	TMemIniFile* ini = NULL;
	TStringList* list = new TStringList();
	int index = 0;
	try {
		ini = new TMemIniFile(FILE_INI);
		list->CommaText = ini->ReadString(INI_SECTION, INI_KEY, "");

		TGameFormOptions options1;
		options1.Sizable = StrToBool(list->Strings[index++]);
		options1.LastPosition = StrToBool(list->Strings[index++]);
		Options = options1;

		TGameBoxOptions options2;
		options2.HighScore = StrToInt(list->Strings[index++]);
		options2.BusyGame = StrToBool(list->Strings[index++]);
		options2.Sound = StrToBool(list->Strings[index++]);
		FGameBox->Options = options2;

		ClientWidth = StrToInt(list->Strings[index++]);
		ClientHeight = StrToInt(list->Strings[index++]);
		FGameBox->Width = ClientWidth;
		FGameBox->Height = ClientHeight;
		int left = StrToInt(list->Strings[index++]);
		int top = StrToInt(list->Strings[index++]);
		if(Options.LastPosition) {
			Left = left;
			Top = top;
		}
	}
	catch(...) {
		DefaultSettings();
	}
	delete list;
	delete ini;
}
//---------------------------------------------------------------------------
void __fastcall TGameForm::SaveSettings()
{
	TMemIniFile* ini = NULL;
	TStringList* list = new TStringList();
	try {
		ini = new TMemIniFile(FILE_INI);

		TGameFormOptions options1 = Options;
		list->Add(BoolToStr(options1.Sizable));
		list->Add(BoolToStr(options1.LastPosition));

		TGameBoxOptions options2 = FGameBox->Options;
		list->Add(IntToStr(options2.HighScore));
		list->Add(BoolToStr(options2.BusyGame));
		list->Add(BoolToStr(options2.Sound));

		list->Add(ClientWidth);
		list->Add(ClientHeight);
		list->Add(Left);
		list->Add(Top);
		list->Add("");

		ini->WriteString(INI_SECTION, INI_KEY, list->CommaText);
		ini->UpdateFile();
	}
	catch(...) {}
	delete list;
	delete ini;
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TGameForm::TGameForm(TComponent* Owner) : TForm(Owner), FGameBox(NULL)
{
	randomize();
	Application->Title = STR_TITLE;
	Caption = STR_TITLE;
	FGameBox = new TGameBox(this);
	FGameBox->Images = ImageList1;
	FGameBox->Parent = this;
	FGameBox->Align = alClient;
	FGameBox->OnChange = GameBoxChange;
	FGameBox->OnChange(FGameBox);
	OptionsChange(this);
	ResizeClient(100);

	// load settings
	LoadSettings();

	// show title
	FGameBox->Title();
}
//---------------------------------------------------------------------------
__fastcall TGameForm::~TGameForm()
{
	// quit
	FGameBox->Quit();

	// save settings
	SaveSettings();
}
//---------------------------------------------------------------------------


