//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TGameMap.h"
using namespace std;
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class TGameMap
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
GAMEWALL __fastcall TGameMap::GetWall(int AX, int AY) const
{
	return FWalls[AX][AY];
}
//---------------------------------------------------------------------------
void __fastcall TGameMap::SetWall(int AX, int AY, GAMEWALL AWall)
{
	FWalls[AX][AY] = AWall;
	AddChange(AX, AY);
}
//---------------------------------------------------------------------------
TGameUnit* __fastcall TGameMap::GetUnit(int AX, int AY) const
{
	return FUnits[AX][AY];
}
//---------------------------------------------------------------------------
void __fastcall TGameMap::SetUnit(int AX, int AY, TGameUnit* AUnit)
{
	FUnits[AX][AY] = AUnit;
	AddChange(AX, AY);
}
//---------------------------------------------------------------------------
int __fastcall TGameMap::GetChangeCount() const
{
	return static_cast<int>(FChanges.size());
}
//---------------------------------------------------------------------------
TPoint __fastcall TGameMap::GetChange(int AIndex) const
{
	return FChanges[AIndex];
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TGameMap::TGameMap(int AWidth, int AHeight) : FWidth(AWidth), FHeight(AHeight)
{
	FWalls = new GAMEWALL*[AWidth];
	FUnits = new TGameUnit**[AWidth];
	FChanged = new bool*[AWidth];
	for(int x = 0; x < FWidth; x++) {
		FWalls[x] = new GAMEWALL[AHeight];
		FUnits[x] = new TGameUnit*[AHeight];
		FChanged[x] = new bool[AHeight];
		for(int y = 0; y < FHeight; y++) {
			FWalls[x][y] = GAMEWALL_NONE;
			FUnits[x][y] = NULL;
			FChanged[x][y] = false;
		}
	}
}
//---------------------------------------------------------------------------
__fastcall TGameMap::~TGameMap()
{
	for(int x = 0; x < FWidth; x++) {
		delete [] FChanged[x];
		delete [] FUnits[x];
		delete [] FWalls[x];
	}
	delete [] FChanged;
	delete [] FUnits;
	delete [] FWalls;
}
//---------------------------------------------------------------------------
bool __fastcall TGameMap::IsEmpty(int AX, int AY) const
{
	return FWalls[AX][AY] == NULL && FUnits[AX][AY] == NULL;
}
//---------------------------------------------------------------------------
bool __fastcall TGameMap::CanGo(int AX, int AY) const
{
	return AX >= 0 && AX < Width && AY >= 0 && AY < Height &&
		Walls[AX][AY] == GAMEWALL_NONE &&
		(Units[AX][AY] == NULL || Units[AX][AY]->Id != TGameUnit::ID_PLAYER);
}
//---------------------------------------------------------------------------
int __fastcall TGameMap::CountWays(int AX, int AY) const
{
	return (CanGo(AX - 1, AY) ? 1 : 0) + (CanGo(AX + 1, AY) ? 1 : 0) +
		(CanGo(AX, AY - 1) ? 1 : 0) + (CanGo(AX, AY + 1) ? 1 : 0);
}
//---------------------------------------------------------------------------
void __fastcall TGameMap::Clear()
{
	for(int x = 0; x < FWidth; x++) {
		for(int y = 0; y < FHeight; y++) {
			ClearWall(x, y);
			ClearUnit(x, y);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TGameMap::ClearWall(int AX, int AY)
{
	Walls[AX][AY] = GAMEWALL_NONE;
	AddChange(AX, AY);
}
//---------------------------------------------------------------------------
void __fastcall TGameMap::ClearUnit(int AX, int AY)
{
	Units[AX][AY] = NULL;
	AddChange(AX, AY);
}
//---------------------------------------------------------------------------
void __fastcall TGameMap::AddChange(int AX, int AY)
{
	if(FChanged[AX][AY]) return;

	FChanged[AX][AY] = true;
	FChanges.push_back(TPoint(AX, AY));
}
//---------------------------------------------------------------------------
void __fastcall TGameMap::ClearChanges()
{
	for(vector<TPoint>::iterator it = FChanges.begin(); it != FChanges.end(); it++)
		FChanged[it->x][it->y] = false;
	FChanges.clear();
}
//---------------------------------------------------------------------------

