//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TGameUnit.h"
#include "TGameField.h"
#include "TGameMap.h"
#include "TGamePad.h"
#include "TGameSound.h"
#include "Common.h"
#pragma package(smart_init)
using namespace std;
//---------------------------------------------------------------------------
// class TGameUnit
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TGameUnit::TGameUnit(ID AId) : FId(AId), FX(0), FY(0), FValid(true), FImage(0)
{
	// do nothing
}
//---------------------------------------------------------------------------
__fastcall TGameUnit::~TGameUnit()
{
	// do nothing
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// class TGamePlayerUnit : public TGameUnit
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TGamePlayerUnit::TGamePlayerUnit(int AIndex) : TGameUnit(ID_PLAYER),
	FIndex(AIndex), FStuck(false), FBack(false), FDX(0), FDY(0), FLastDX(0), FLastDY(0),
	FGrowth(0), FScore(0)
{
	Image = Index == 0 ? 10 : 20;
}
//---------------------------------------------------------------------------
__fastcall TGamePlayerUnit::~TGamePlayerUnit()
{
	// do nothing
}
//---------------------------------------------------------------------------
void __fastcall TGamePlayerUnit::Move(TGameField* AField, TGamePad** APads, TGameSound* ASound)
{
	// check conditions
	if(Stuck || Back) return;

	// do animation
	Image = (random(60) == 0 ? 1 : 0) + (Index == 0 ? 10 : 20);
	AField->Map->AddChange(X, Y);

	// check stucked
	if(AField->Map->CountWays(X, Y) <= 0) {
		FStuck = true;
		Image = 2 + (Index == 0 ? 10 : 20);
		return;
	}

	// change direction
	{
		// prepare table
		int dx, dy; dx = dy = 0;
		for(int i = 0; i < 2; i++) {
			for(int j = 0; j < PADIDCOUNT; j++) {
				switch(i == 0 ? APads[Index]->PressingOrder(j) : APads[Index]->PressedOrder(j)) {
					case PADID_LEFT: dx = -1; break;
					case PADID_RIGHT: dx = 1; break;
					case PADID_UP: dy = -1; break;
					case PADID_DOWN: dy = 1; break;
					default: continue;
				}
				i = 2;
				j = PADIDCOUNT;
			}
		}
		if(dx == 0 && dy == 0 && FDX == 0 && FDY == 0) return;
		int dxs[] = {dx, FDX, FLastDX, 0, -1, 1, 0, 0};
		int dys[] = {dy, FDY, 0, FLastDY, 0, 0, -1, 1};
		int count = sizeof(dxs) / sizeof(int);

		// shuffle table
		int offset = 4;
		int r, n;
		for(int i = 0; i < 4; i++) {
			r = offset + random(4);
			n = dxs[offset + i]; dxs[offset + i] = dxs[r]; dxs[r] = n;
			n = dys[offset + i]; dys[offset + i] = dys[r]; dys[r] = n;
		}

		// change direction
		bool corrision = false;
		for(int i = 0; i < count; i++) {
			if(dxs[i] == 0 && dys[i] == 0) {
				continue;
			}
			else if(!AField->Map->CanGo(X + dxs[i], Y + dys[i])) {
				corrision = true;
				continue;
			}
			if(dxs[i] != FDX && dys[i] != FDY && corrision &&
				AField->Map->CountWays(X, Y) >= 2) ASound->Play(SOUNDID_CORRISION);
			FDX = dxs[i];
			FDY = dys[i];
			break;
		}
	}
	if(FDX != 0) FLastDX = FDX;
	if(FDY != 0) FLastDY = FDY;

	// move player
	if(AField->Map->Units[X][Y] == NULL || AField->Map->Units[X][Y]->Id != ID_HOME)
		AField->Map->ClearUnit(X, Y);
	int lastX = X;
	int lastY = Y;
	X = X + FDX;
	Y = Y + FDY;
	TGameUnit* unit = AField->Map->Units[X][Y];
	if(unit == NULL) {
		AField->Map->Units[X][Y] = this;
	}
	else if(unit->Id == ID_TARGET) {
		AField->Map->Units[X][Y] = this;

		unit->Valid = false;
		switch(dynamic_cast<TGameTargetUnit*>(unit)->Type) {
			case TGameTargetUnit::TYPE_CAKE:
				FScore++;
				FGrowth++;
				AField->Status = "Got a Baumkuchen";
				ASound->Play(SOUNDID_CAKE);
				break;

			case TGameTargetUnit::TYPE_STAMP:
				AField->Status = "Got a Stamp";
				ASound->Play(SOUNDID_STAMP);
				break;

			case TGameTargetUnit::TYPE_GELATO:
				FScore++;
				AField->Status = "Got a Gelato";
				ASound->Play(SOUNDID_GELATO);
				break;
		}
	}
	else if(unit->Id == ID_HOME) {
		FBack = true;
		FScore += FTails.size();
	}

	// move tails
	if(FTails.size() > 0 || FGrowth > 0) {
		AField->Map->Walls[lastX][lastY] = GAMEWALL_TAIL;
		FTails.push_front(TPoint(lastX, lastY));
	}
	if(FTails.size() > 0 && FGrowth <= 0) {
		AField->Map->Walls[FTails.back().x][FTails.back().y] = GAMEWALL_NONE;
		FTails.pop_back();
	}
	if(FGrowth > 0) FGrowth--;
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// class TGameTargetUnit : public TGameUnit
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TGameTargetUnit::TGameTargetUnit(TYPE AType) : TGameUnit(ID_TARGET),
	FType(AType), FTimeLimit(0)
{
	switch(FType) {
		case TYPE_CAKE:
			Image = 30;
			FTimeLimit = 50 + random(25);
			break;

		case TYPE_STAMP:
			Image = 31;
			FTimeLimit = 25 + random(25);
			break;

		case TYPE_GELATO:
			Image = 32;
			FTimeLimit = 50 + random(25);
			break;
	}
}
//---------------------------------------------------------------------------
__fastcall TGameTargetUnit::~TGameTargetUnit()
{
	// do nothing
}
//---------------------------------------------------------------------------
void __fastcall TGameTargetUnit::Move(TGameField* AField, TGamePad** APads, TGameSound* ASound)
{
	if(--FTimeLimit > 0) return;

	Valid = false;
	AField->Map->ClearUnit(X, Y);

	if(FType == TYPE_CAKE) {
		TGameTargetUnit* unit = new TGameTargetUnit(TYPE_STAMP);
		unit->X = X;
		unit->Y = Y;
		AField->Map->Units[X][Y] = unit;
		AField->RegisterUnit(unit);
	}
	else if(FType == TYPE_STAMP) {
		AField->Map->Walls[X][Y] = GAMEWALL_TREE;
	}
	else if(FType == TYPE_GELATO) {
		// do nothing
	}
}
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// class TGameHomeUnit : public TGameUnit
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TGameHomeUnit::TGameHomeUnit() : TGameUnit(ID_HOME)
{
	Image = 40;
}
//---------------------------------------------------------------------------
__fastcall TGameHomeUnit::~TGameHomeUnit()
{
	// do nothing
}
//---------------------------------------------------------------------------
void __fastcall TGameHomeUnit::Move(TGameField* AField, TGamePad** APads, TGameSound* ASound)
{
	// do nothing
}
//---------------------------------------------------------------------------

