//---------------------------------------------------------------------------
#ifndef TGameUnitH
#define TGameUnitH
//---------------------------------------------------------------------------
#include <list>
#include <Classes.hpp>
//---------------------------------------------------------------------------
enum GAMEWALL {GAMEWALL_NONE, GAMEWALL_TREE, GAMEWALL_BLOCK, GAMEWALL_TAIL};
//---------------------------------------------------------------------------
class TGameField;
class TGamePad;
class TGameSound;
//---------------------------------------------------------------------------
class TGameUnit
{
public:
	enum ID {ID_PLAYER, ID_TARGET, ID_HOME};

private:
	ID FId;
	int FX, FY;
	bool FValid;
	int FImage;

public:
	__fastcall TGameUnit(ID AId);
	__fastcall virtual ~TGameUnit();

	void __fastcall virtual Move(TGameField* AField, TGamePad** APads, TGameSound* ASound) = 0;

	__property ID Id = {read=FId};
	__property int X = {read=FX, write=FX};
	__property int Y = {read=FY, write=FY};
	__property bool Valid = {read=FValid, write=FValid};
	__property int Image = {read=FImage, write=FImage};
};
//---------------------------------------------------------------------------
class TGamePlayerUnit : public TGameUnit
{
private:
	int FIndex;
	bool FStuck;
	bool FBack;
	int FDX, FDY;
	int FLastDX, FLastDY;
	std::list<TPoint> FTails;
	int FGrowth;
	int FScore;

public:
	__fastcall TGamePlayerUnit(int AIndex);
	__fastcall ~TGamePlayerUnit();

	void __fastcall Move(TGameField* AField, TGamePad** APads, TGameSound* ASound);

	__property int Index = {read=FIndex};
	__property bool Stuck = {read=FStuck};
	__property bool Back = {read=FBack};
	__property int Score = {read=FScore};
};
//---------------------------------------------------------------------------
class TGameTargetUnit : public TGameUnit
{
public:
	enum TYPE {TYPE_CAKE, TYPE_STAMP, TYPE_GELATO};

private:
	TYPE FType;
	int FTimeLimit;

public:
	__fastcall TGameTargetUnit(TYPE AType);
	__fastcall ~TGameTargetUnit();

	void __fastcall Move(TGameField* AField, TGamePad** APads, TGameSound* ASound);

	__property TYPE Type = {read=FType};
};
//---------------------------------------------------------------------------
class TGameHomeUnit : public TGameUnit
{
private:

public:
	__fastcall TGameHomeUnit();
	__fastcall ~TGameHomeUnit();

	void __fastcall Move(TGameField* AField, TGamePad** APads, TGameSound* ASound);
};
//---------------------------------------------------------------------------
#endif

